@@ -5,7 +5,12 @@ from functools import lru_cache
 
 import orjson
 
-from langgraph_api.config import LANGGRAPH_AUTH, LANGGRAPH_AUTH_TYPE, MOUNT_PREFIX
+from langgraph_api.config import (
+    HTTP_CONFIG,
+    LANGGRAPH_AUTH,
+    LANGGRAPH_AUTH_TYPE,
+    MOUNT_PREFIX,
+)
 from langgraph_api.graph import GRAPHS
 from langgraph_api.validation import openapi
 
@@ -80,6 +85,13 @@ def get_openapi_spec() -> str:
         final = merge_openapi_specs(openapi, CUSTOM_OPENAPI_SPEC)
     if MOUNT_PREFIX:
         final["servers"] = [{"url": MOUNT_PREFIX}]
+
+    MCP_ENABLED = HTTP_CONFIG is None or not HTTP_CONFIG.get("disable_mcp")
+
+    if not MCP_ENABLED:
+        # Remove the MCP paths from the OpenAPI spec
+        final["paths"].pop("/mcp/", None)
+
     return orjson.dumps(final)
 
 
