@@ -39,7 +39,10 @@ async def create_run(request: ApiRequest):
             payload,
             request.headers,
         )
-    return ApiResponse(run)
+    return ApiResponse(
+        run,
+        headers={"Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}"},
+    )
 
 
 @retry_db
@@ -53,7 +56,10 @@ async def create_stateless_run(request: ApiRequest):
             payload,
             request.headers,
         )
-    return ApiResponse(run)
+    return ApiResponse(
+        run,
+        headers={"Content-Location": f"/runs/{run['run_id']}"},
+    )
 
 
 async def create_stateless_run_batch(request: ApiRequest):
@@ -107,8 +113,12 @@ async def stream_run(
             thread_id=thread_id,
             cancel_on_disconnect=on_disconnect == "cancel",
             stream_mode=await sub,
+            last_event_id=None,
         ),
-        headers={"Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream"},
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream",
+            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+        },
     )
 
 
@@ -143,9 +153,11 @@ async def stream_run_stateless(
             ignore_404=True,
             cancel_on_disconnect=on_disconnect == "cancel",
             stream_mode=await sub,
+            last_event_id=None,
         ),
         headers={
-            "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/stream"
+            "Location": f"/runs/{run['run_id']}/stream",
+            "Content-Location": f"/runs/{run['run_id']}",
         },
     )
 
@@ -182,7 +194,7 @@ async def wait_run(request: ApiRequest):
                 run["run_id"], thread_id=run["thread_id"], stream_mode=await sub
             )
         ) as stream:
-            async for mode, chunk in stream:
+            async for mode, chunk, _ in stream:
                 if mode == b"values":
                     vchunk = chunk
                 elif mode == b"error":
@@ -216,7 +228,10 @@ async def wait_run(request: ApiRequest):
     return StreamingResponse(
         body(),
         media_type="application/json",
-        headers={"Location": f"/threads/{thread_id}/runs/{run['run_id']}/join"},
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/join",
+            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+        },
     )
 
 
@@ -251,7 +266,7 @@ async def wait_run_stateless(request: ApiRequest):
                 run["run_id"], thread_id=run["thread_id"], stream_mode=await sub
             )
         ) as stream:
-            async for mode, chunk in stream:
+            async for mode, chunk, _ in stream:
                 if mode == b"values":
                     vchunk = chunk
                 elif mode == b"error":
@@ -280,12 +295,15 @@ async def wait_run_stateless(request: ApiRequest):
     return StreamingResponse(
         body(),
         media_type="application/json",
-        headers={"Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join"},
+        headers={
+            "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join",
+            "Content-Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}",
+        },
     )
 
 
 @retry_db
-async def list_runs_http(
+async def list_runs(
     request: ApiRequest,
 ):
     """List all runs for a thread."""
@@ -312,7 +330,7 @@ async def list_runs_http(
 
 
 @retry_db
-async def get_run_http(request: ApiRequest):
+async def get_run(request: ApiRequest):
     """Get a run by ID."""
     thread_id = request.path_params["thread_id"]
     run_id = request.path_params["run_id"]
@@ -349,7 +367,7 @@ async def join_run(request: ApiRequest):
 
 
 @retry_db
-async def join_run_stream_endpoint(request: ApiRequest):
+async def join_run_stream(request: ApiRequest):
     """Wait for a run to finish."""
     thread_id = request.path_params["thread_id"]
     run_id = request.path_params["run_id"]
@@ -358,13 +376,15 @@ async def join_run_stream_endpoint(request: ApiRequest):
     validate_uuid(thread_id, "Invalid thread ID: must be a UUID")
     validate_uuid(run_id, "Invalid run ID: must be a UUID")
     stream_mode = request.query_params.get("stream_mode") or None
+    last_event_id = request.headers.get("last-event-id") or None
     return EventSourceResponse(
         Runs.Stream.join(
             run_id,
             thread_id=thread_id,
             cancel_on_disconnect=cancel_on_disconnect,
             stream_mode=stream_mode,
-        )
+            last_event_id=last_event_id,
+        ),
     )
 
 
@@ -541,11 +561,11 @@ runs_routes = [
     ApiRoute("/threads/{thread_id}/runs/{run_id}/join", join_run, methods=["GET"]),
     ApiRoute(
         "/threads/{thread_id}/runs/{run_id}/stream",
-        join_run_stream_endpoint,
+        join_run_stream,
         methods=["GET"],
     ),
     ApiRoute("/threads/{thread_id}/runs/{run_id}/cancel", cancel_run, methods=["POST"]),
-    ApiRoute("/threads/{thread_id}/runs/{run_id}", get_run_http, methods=["GET"]),
+    ApiRoute("/threads/{thread_id}/runs/{run_id}", get_run, methods=["GET"]),
     ApiRoute("/threads/{thread_id}/runs/{run_id}", delete_run, methods=["DELETE"]),
     ApiRoute("/threads/{thread_id}/runs/stream", stream_run, methods=["POST"]),
     ApiRoute("/threads/{thread_id}/runs/wait", wait_run, methods=["POST"]),
@@ -557,7 +577,7 @@ runs_routes = [
         if config.FF_CRONS_ENABLED and plus_features_enabled()
         else None
     ),
-    ApiRoute("/threads/{thread_id}/runs", list_runs_http, methods=["GET"]),
+    ApiRoute("/threads/{thread_id}/runs", list_runs, methods=["GET"]),
     (
         ApiRoute("/runs/crons/{cron_id}", delete_cron, methods=["DELETE"])
         if config.FF_CRONS_ENABLED and plus_features_enabled()
