@@ -6,6 +6,7 @@ from functools import partial
 from typing import Any, Generic, TypeVar
 
 import structlog
+from langgraph.utils.future import chain_future
 
 T = TypeVar("T")
 
@@ -19,6 +20,12 @@ def set_event_loop(loop: asyncio.AbstractEventLoop) -> None:
     _MAIN_LOOP = loop
 
 
+def get_event_loop() -> asyncio.AbstractEventLoop:
+    if _MAIN_LOOP is None:
+        raise RuntimeError("No event loop set")
+    return _MAIN_LOOP
+
+
 async def sleep_if_not_done(delay: float, done: asyncio.Event) -> None:
     try:
         await asyncio.wait_for(done.wait(), delay)
@@ -118,6 +125,16 @@ def run_coroutine_threadsafe(
     return future
 
 
+def call_soon_in_main_loop(coro: Coroutine[Any, Any, T]) -> asyncio.Future[T]:
+    """Run a coroutine in the main event loop."""
+    if _MAIN_LOOP is None:
+        raise RuntimeError("No event loop set")
+    main_loop_fut = asyncio.ensure_future(coro, loop=_MAIN_LOOP)
+    this_loop_fut = asyncio.get_running_loop().create_future()
+    _MAIN_LOOP.call_soon_threadsafe(chain_future, main_loop_fut, this_loop_fut)
+    return this_loop_fut
+
+
 class SimpleTaskGroup(AbstractAsyncContextManager["SimpleTaskGroup"]):
     """An async task group that can be configured to wait and/or cancel tasks on exit.
 
