@@ -18,6 +18,11 @@ class CorsConfig(TypedDict, total=False):
     max_age: int
 
 
+class ConfigurableHeaders(TypedDict):
+    includes: list[str] | None
+    excludes: list[str] | None
+
+
 class HttpConfig(TypedDict, total=False):
     app: str
     """Import path for a custom Starlette/FastAPI app to mount"""
@@ -39,6 +44,7 @@ class HttpConfig(TypedDict, total=False):
     """Disable /mcp routes"""
     mount_prefix: str
     """Prefix for mounted routes. E.g., "/my-deployment/api"."""
+    configurable_headers: ConfigurableHeaders | None
 
 
 class ThreadTTLConfig(TypedDict, total=False):
@@ -103,6 +109,7 @@ class TTLConfig(TypedDict, total=False):
 
 
 class StoreConfig(TypedDict, total=False):
+    path: str
     index: IndexConfig
     ttl: TTLConfig
 
@@ -141,6 +148,7 @@ STATS_INTERVAL_SECS = env("STATS_INTERVAL_SECS", cast=int, default=60)
 
 DATABASE_URI = env("DATABASE_URI", cast=str, default=getenv("POSTGRES_URI", undefined))
 MIGRATIONS_PATH = env("MIGRATIONS_PATH", cast=str, default="/storage/migrations")
+POSTGRES_POOL_MAX_SIZE = env("LANGGRAPH_POSTGRES_POOL_MAX_SIZE", cast=int, default=150)
 
 
 def _get_encryption_key(key_str: str | None):
@@ -221,7 +229,8 @@ CORS_CONFIG: CorsConfig | None = env("CORS_CONFIG", cast=_parse_json, default=No
 }
 """
 if CORS_CONFIG is not None and CORS_ALLOW_ORIGINS != "*":
-    raise ValueError("CORS_CONFIG and CORS_ALLOW_ORIGINS cannot be set together")
+    if CORS_CONFIG.get("allow_origins") is None:
+        CORS_CONFIG["allow_origins"] = CORS_ALLOW_ORIGINS
 
 # queue
 
@@ -229,6 +238,9 @@ BG_JOB_HEARTBEAT = 120  # seconds
 BG_JOB_INTERVAL = 30  # seconds
 BG_JOB_MAX_RETRIES = 3
 BG_JOB_ISOLATED_LOOPS = env("BG_JOB_ISOLATED_LOOPS", cast=bool, default=False)
+BG_JOB_SHUTDOWN_GRACE_PERIOD_SECS = env(
+    "BG_JOB_SHUTDOWN_GRACE_PERIOD_SECS", cast=int, default=3600
+)
 
 
 def _parse_thread_ttl(value: str | None) -> ThreadTTLConfig | None:
