@@ -39,6 +39,7 @@ from starlette.exceptions import HTTPException
 from starlette.requests import HTTPConnection, Request
 from starlette.routing import Route
 
+from langgraph_api import store as api_store
 from langgraph_api.auth.custom import DotDict, ProxyUser
 from langgraph_api.config import LANGGRAPH_AUTH_TYPE
 from langgraph_api.js.base import BaseRemotePregel
@@ -70,6 +71,12 @@ _client = httpx.AsyncClient(
 )
 
 
+def _snapshot_defaults():
+    if not hasattr(StateSnapshot, "interrupts"):
+        return {}
+    return {"interrupts": tuple()}
+
+
 def default_command(obj):
     if isinstance(obj, Send):
         return {"node": obj.node, "args": obj.arg}
@@ -250,6 +257,8 @@ class RemotePregel(BaseRemotePregel):
             item.get("createdAt"),
             item.get("parentConfig"),
             _convert_tasks(item.get("tasks", [])),
+            # TODO: add handling of interrupts when multiple resumes land in JS
+            **_snapshot_defaults(),
         )
 
     async def aget_state(
@@ -315,26 +324,30 @@ class RemotePregel(BaseRemotePregel):
         *,
         xray: int | bool = False,
     ) -> dict[str, Any]:
-        raise Exception("Not implemented")
+        raise NotImplementedError()
 
     def get_input_schema(self, config: RunnableConfig | None = None) -> type[BaseModel]:
-        raise Exception("Not implemented")
+        raise NotImplementedError()
 
     def get_output_schema(
         self, config: RunnableConfig | None = None
     ) -> type[BaseModel]:
-        raise Exception("Not implemented")
+        raise NotImplementedError()
 
     def config_schema(self) -> type[BaseModel]:
-        raise Exception("Not implemented")
+        raise NotImplementedError()
 
     async def invoke(self, input: Any, config: RunnableConfig | None = None):
-        raise Exception("Not implemented")
+        raise NotImplementedError()
 
     def copy(self, update: dict[str, Any] | None = None) -> Self:
         attrs = {**self.__dict__, **(update or {})}
         return self.__class__(**attrs)
 
+    async def fetch_nodes_executed(self):
+        result = await _client_invoke("getNodesExecuted", {"graph_id": self.graph_id})
+        return result["nodesExecuted"]
+
 
 async def run_js_process(paths_str: str, watch: bool = False):
     # check if tsx is available
@@ -346,9 +359,22 @@ async def run_js_process(paths_str: str, watch: bool = False):
     attempt = 0
     while not asyncio.current_task().cancelled():
         client_file = os.path.join(os.path.dirname(__file__), "client.mts")
-        args = ("tsx", client_file)
-        if watch:
-            args = ("tsx", "watch", client_file, "--skip-schema-cache")
+        client_preload_file = os.path.join(
+            os.path.dirname(__file__), "src", "preload.mjs"
+        )
+
+        args = (
+            (
+                "tsx",
+                "watch",
+                "--import",
+                client_preload_file,
+                client_file,
+                "--skip-schema-cache",
+            )
+            if watch
+            else ("tsx", "--import", client_preload_file, client_file)
+        )
         try:
             process = await asyncio.create_subprocess_exec(
                 *args,
@@ -454,10 +480,8 @@ def _get_passthrough_checkpointer(conn: AsyncConnectionProto):
     return checkpointer
 
 
-def _get_passthrough_store():
-    from langgraph_runtime.store import Store
-
-    return Store()
+async def _get_passthrough_store():
+    return await api_store.get_store()
 
 
 # Setup a HTTP server on top of CHECKPOINTER_SOCKET unix socket
@@ -555,7 +579,7 @@ async def run_remote_checkpointer():
             else:
                 raise ValueError(f"Unknown operation type: {op}")
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         results = await store.abatch(processed_operations)
 
         # Handle potentially undefined or non-dict results
@@ -594,7 +618,7 @@ async def run_remote_checkpointer():
 
         namespaces = namespaces_str.split(".")
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         result = await store.aget(namespaces, key)
 
         return result
@@ -607,7 +631,7 @@ async def run_remote_checkpointer():
         value = payload["value"]
         index = payload.get("index")
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         await store.aput(namespace, key, value, index=index)
 
         return {"success": True}
@@ -620,7 +644,7 @@ async def run_remote_checkpointer():
         offset = payload.get("offset", 0)
         query = payload.get("query")
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         result = await store.asearch(
             namespace_prefix, filter=filter, limit=limit, offset=offset, query=query
         )
@@ -633,7 +657,7 @@ async def run_remote_checkpointer():
         namespace = tuple(payload["namespace"])
         key = payload["key"]
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         await store.adelete(namespace, key)
 
         return {"success": True}
@@ -646,7 +670,7 @@ async def run_remote_checkpointer():
         limit = payload.get("limit", 100)
         offset = payload.get("offset", 0)
 
-        store = _get_passthrough_store()
+        store = await _get_passthrough_store()
         result = await store.alist_namespaces(
             prefix=prefix,
             suffix=suffix,
@@ -843,12 +867,14 @@ async def handle_js_auth_event(
                 "resource": ctx.resource,
                 "action": ctx.action,
                 "value": value,
-                "context": {
-                    "user": cast(DotDict, ctx.user).dict(),
-                    "scopes": ctx.permissions,
-                }
-                if ctx
-                else None,
+                "context": (
+                    {
+                        "user": cast(DotDict, ctx.user).dict(),
+                        "scopes": ctx.permissions,
+                    }
+                    if ctx
+                    else None
+                ),
             }
         ),
     )
