@@ -1,4 +1,6 @@
 import asyncio
+import functools
+import re
 import urllib.parse
 import uuid
 from collections.abc import Mapping, Sequence
@@ -75,6 +77,8 @@ class RunCreateDict(TypedDict):
     """
     stream_subgraphs: bool | None
     """Stream output from subgraphs. By default, streams only the top graph."""
+    stream_resumable: bool | None
+    """Whether to persist the stream chunks in order to resume the stream later."""
     feedback_keys: list[str] | None
     """Pass one or more feedback_keys if you want to request short-lived signed URLs
     for submitting feedback to LangSmith with this key for this run."""
@@ -159,6 +163,85 @@ LANGSMITH_TAGS = "langsmith-tags"
 LANGSMITH_PROJECT = "langsmith-project"
 
 
+def translate_pattern(pat: str) -> re.Pattern[str]:
+    """Translate a pattern to regex, supporting only literals and * wildcards to avoid RE DoS."""
+    res = []
+    i = 0
+    n = len(pat)
+
+    while i < n:
+        c = pat[i]
+        i += 1
+
+        if c == "*":
+            res.append(".*")
+        else:
+            res.append(re.escape(c))
+
+    pattern = "".join(res)
+    return re.compile(rf"(?s:{pattern})\Z")
+
+
+@functools.lru_cache(maxsize=1)
+def get_header_patterns() -> tuple[
+    list[re.Pattern[str] | None], list[re.Pattern[str] | None]
+]:
+    from langgraph_api import config
+
+    if not config.HTTP_CONFIG:
+        return None, None
+    configurable = config.HTTP_CONFIG.get("configurable_headers")
+    if not configurable:
+        return None, None
+    header_includes = configurable.get("includes") or []
+    include_patterns = []
+    for include in header_includes:
+        include_patterns.append(translate_pattern(include))
+    header_excludes = configurable.get("excludes") or []
+    exclude_patterns = []
+    for exclude in header_excludes:
+        exclude_patterns.append(translate_pattern(exclude))
+    return include_patterns, exclude_patterns
+
+
+def get_configurable_headers(headers: dict[str, str]) -> dict[str, str]:
+    configurable = {}
+    include_patterns, exclude_patterns = get_header_patterns()
+    for key, value in headers.items():
+        # First handle tracing stuff; not configurable
+        if key == "langsmith-trace":
+            configurable[key] = value
+            if baggage := headers.get("baggage"):
+                for item in baggage.split(","):
+                    key, value = item.split("=")
+                    if key == LANGSMITH_METADATA and key not in configurable:
+                        configurable[key] = orjson.loads(urllib.parse.unquote(value))
+                    elif key == LANGSMITH_TAGS:
+                        configurable[key] = urllib.parse.unquote(value).split(",")
+                    elif key == LANGSMITH_PROJECT:
+                        configurable[key] = urllib.parse.unquote(value)
+        # Then handle overridable behavior
+        if exclude_patterns and any(pattern.match(key) for pattern in exclude_patterns):
+            continue
+        if include_patterns and any(pattern.match(key) for pattern in include_patterns):
+            configurable[key] = value
+            continue
+
+        # Then handle default behavior
+        if key.startswith("x-"):
+            if key in (
+                "x-api-key",
+                "x-tenant-id",
+                "x-service-key",
+            ):
+                continue
+            configurable[key] = value
+
+        elif key == "user-agent":
+            configurable[key] = value
+    return configurable
+
+
 async def create_valid_run(
     conn: AsyncConnectionProto,
     thread_id: str | None,
@@ -167,6 +250,7 @@ async def create_valid_run(
     barrier: asyncio.Barrier | None = None,
     run_id: UUID | None = None,
 ) -> Run:
+    request_id = headers.get("x-request-id")  # Will be null in the crons scheduler.
     (
         assistant_id,
         thread_id,
@@ -197,28 +281,7 @@ async def create_valid_run(
         configurable["checkpoint_id"] = str(checkpoint_id)
     if checkpoint := payload.get("checkpoint"):
         configurable.update(checkpoint)
-    for key, value in headers.items():
-        if key.startswith("x-"):
-            if key in (
-                "x-api-key",
-                "x-tenant-id",
-                "x-service-key",
-            ):
-                continue
-            configurable[key] = value
-        elif key == "langsmith-trace":
-            configurable[key] = value
-            if baggage := headers.get("baggage"):
-                for item in baggage.split(","):
-                    key, value = item.split("=")
-                    if key == LANGSMITH_METADATA and key not in configurable:
-                        configurable[key] = orjson.loads(urllib.parse.unquote(value))
-                    elif key == LANGSMITH_TAGS:
-                        configurable[key] = urllib.parse.unquote(value).split(",")
-                    elif key == LANGSMITH_PROJECT:
-                        configurable[key] = urllib.parse.unquote(value)
-        elif key == "user-agent":
-            configurable[key] = value
+    configurable.update(get_configurable_headers(headers))
     ctx = get_auth_ctx()
     if ctx:
         user = ctx.user
@@ -228,6 +291,8 @@ async def create_valid_run(
         configurable["langgraph_auth_permissions"] = ctx.permissions
     else:
         user_id = None
+    if not configurable.get("langgraph_request_id"):
+        configurable["langgraph_request_id"] = request_id
     run_coro = Runs.put(
         conn,
         assistant_id,
@@ -242,6 +307,8 @@ async def create_valid_run(
             "feedback_keys": payload.get("feedback_keys"),
             "temporary": temporary,
             "subgraphs": payload.get("stream_subgraphs", False),
+            "resumable": payload.get("stream_resumable", False),
+            "checkpoint_during": payload.get("checkpoint_during", True),
         },
         metadata=payload.get("metadata"),
         status="pending",
