@@ -1,3 +1,14 @@
+# ruff: noqa: E402
+import os
+
+if not (
+    (disable_truststore := os.getenv("DISABLE_TRUSTSTORE"))
+    and disable_truststore.lower() == "true"
+):
+    import truststore  # noqa: F401
+
+    truststore.inject_into_ssl()  # noqa: F401
+
 import asyncio
 import http.server
 import json
@@ -19,6 +30,13 @@ async def healthcheck_server():
     ok_len = str(len(ok))
 
     class HealthHandler(http.server.SimpleHTTPRequestHandler):
+        def log_message(self, format, *args):
+            # Skip logging for /ok endpoint
+            if self.path == "/ok":
+                return
+            # Log other requests normally
+            super().log_message(format, *args)
+
         def do_GET(self):
             if self.path == "/ok":
                 self.send_response(200)
