@@ -9,6 +9,7 @@ import structlog
 from langchain_core.messages import (
     BaseMessage,
     BaseMessageChunk,
+    convert_to_messages,
     message_chunk_to_message,
 )
 from langchain_core.runnables.config import run_in_executor
@@ -22,6 +23,7 @@ from langgraph.pregel.debug import CheckpointPayload, TaskResultPayload
 from pydantic import ValidationError
 from pydantic.v1 import ValidationError as ValidationErrorLegacy
 
+from langgraph_api import store as api_store
 from langgraph_api.asyncio import ValueEvent, wait_if_not_done
 from langgraph_api.command import map_cmd
 from langgraph_api.graph import get_graph
@@ -32,7 +34,6 @@ from langgraph_api.serde import json_dumpb
 from langgraph_api.utils import AsyncConnectionProto
 from langgraph_runtime.checkpoint import Checkpointer
 from langgraph_runtime.ops import Runs
-from langgraph_runtime.store import Store
 
 logger = structlog.stdlib.get_logger(__name__)
 
@@ -86,6 +87,7 @@ async def astream_state(
     # extract args from run
     kwargs = run["kwargs"].copy()
     kwargs.pop("webhook", None)
+    kwargs.pop("resumable", False)
     subgraphs = kwargs.get("subgraphs", False)
     temporary = kwargs.pop("temporary", False)
     config = kwargs.pop("config")
@@ -93,7 +95,7 @@ async def astream_state(
         get_graph(
             config["configurable"]["graph_id"],
             config,
-            store=Store(),
+            store=(await api_store.get_store()),
             checkpointer=None if temporary else Checkpointer(conn),
         )
     )
@@ -116,9 +118,10 @@ async def astream_state(
     config["metadata"]["langgraph_host"] = HOST
     config["metadata"]["langgraph_api_url"] = USER_API_URL
     # attach node counter
-    if not isinstance(graph, BaseRemotePregel):
+    is_remote_pregel = isinstance(graph, BaseRemotePregel)
+    if not is_remote_pregel:
         config["configurable"]["__pregel_node_finished"] = incr_nodes
-        # TODO add node tracking for JS graphs
+
     # attach run_id to config
     # for attempts beyond the first, use a fresh, unique run_id
     config = {**config, "run_id": run["run_id"]} if attempt == 1 else config
@@ -162,6 +165,7 @@ async def astream_state(
                         ns, mode, chunk = event["data"]["chunk"]
                     else:
                         mode, chunk = event["data"]["chunk"]
+                        ns = None
                     # --- begin shared logic with astream ---
                     if mode == "debug":
                         if chunk["type"] == "checkpoint":
@@ -171,9 +175,16 @@ async def astream_state(
                             on_task_result(chunk["payload"])
                     if mode == "messages":
                         if "messages-tuple" in stream_mode:
-                            yield "messages", chunk
+                            if subgraphs and ns:
+                                yield f"messages|{'|'.join(ns)}", chunk
+                            else:
+                                yield "messages", chunk
                         else:
-                            msg, meta = cast(tuple[BaseMessage, dict[str, Any]], chunk)
+                            msg, meta = cast(
+                                tuple[BaseMessage | dict, dict[str, Any]], chunk
+                            )
+                            if isinstance(msg, dict):
+                                msg = convert_to_messages([msg])[0]
                             if msg.id in messages:
                                 messages[msg.id] += msg
                             else:
@@ -213,6 +224,7 @@ async def astream_state(
                     ns, mode, chunk = event
                 else:
                     mode, chunk = event
+                    ns = None
                 # --- begin shared logic with astream_events ---
                 if mode == "debug":
                     if chunk["type"] == "checkpoint":
@@ -222,9 +234,16 @@ async def astream_state(
                         on_task_result(chunk["payload"])
                 if mode == "messages":
                     if "messages-tuple" in stream_mode:
-                        yield "messages", chunk
+                        if subgraphs and ns:
+                            yield f"messages|{'|'.join(ns)}", chunk
+                        else:
+                            yield "messages", chunk
                     else:
-                        msg, meta = cast(tuple[BaseMessage, dict[str, Any]], chunk)
+                        msg, meta = cast(
+                            tuple[BaseMessage | dict, dict[str, Any]], chunk
+                        )
+                        if isinstance(msg, dict):
+                            msg = convert_to_messages([msg])[0]
                         if msg.id in messages:
                             messages[msg.id] += msg
                         else:
@@ -244,6 +263,14 @@ async def astream_state(
                     else:
                         yield mode, chunk
                 # --- end shared logic with astream_events ---
+    if is_remote_pregel:
+        # increament the remote runs
+        try:
+            nodes_executed = await graph.fetch_nodes_executed()
+            incr_nodes(None, incr=nodes_executed)
+        except Exception as e:
+            logger.warning(f"Failed to fetch nodes executed for {graph.graph_id}: {e}")
+
     # Get feedback URLs
     if feedback_keys:
         feedback_urls = await run_in_executor(
@@ -252,19 +279,25 @@ async def astream_state(
         yield "feedback", feedback_urls
 
 
-async def consume(stream: AnyStream, run_id: str) -> None:
+async def consume(stream: AnyStream, run_id: str, resumable: bool = False) -> None:
     async with aclosing(stream):
         try:
             async for mode, payload in stream:
                 await Runs.Stream.publish(
-                    run_id, mode, await run_in_executor(None, json_dumpb, payload)
+                    run_id,
+                    mode,
+                    await run_in_executor(None, json_dumpb, payload),
+                    resumable=resumable,
                 )
         except Exception as e:
             g = e
             if isinstance(e, ExceptionGroup):
                 e = e.exceptions[0]
             await Runs.Stream.publish(
-                run_id, "error", await run_in_executor(None, json_dumpb, e)
+                run_id,
+                "error",
+                await run_in_executor(None, json_dumpb, e),
+                resumable=resumable,
             )
             raise e from g
 
