@@ -8,23 +8,18 @@ import structlog
 from langgraph.pregel.debug import CheckpointPayload, TaskResultPayload
 from starlette.exceptions import HTTPException
 
+import langgraph_api.logging as lg_logging
 from langgraph_api.auth.custom import SimpleUser, normalize_user
 from langgraph_api.config import (
     BG_JOB_ISOLATED_LOOPS,
     BG_JOB_MAX_RETRIES,
     BG_JOB_TIMEOUT_SECS,
 )
-from langgraph_api.errors import (
-    UserInterrupt,
-    UserRollback,
-)
+from langgraph_api.errors import UserInterrupt, UserRollback
 from langgraph_api.js.errors import RemoteException
 from langgraph_api.metadata import incr_runs
 from langgraph_api.schema import Run
-from langgraph_api.stream import (
-    astream_state,
-    consume,
-)
+from langgraph_api.stream import astream_state, consume
 from langgraph_api.utils import set_auth_ctx, with_user
 from langgraph_runtime.database import connect
 from langgraph_runtime.ops import Runs, Threads
@@ -88,12 +83,21 @@ async def worker(
         Runs.enter(run_id, main_loop) as done,
     ):
         temporary = run["kwargs"].get("temporary", False)
+        resumable = run["kwargs"].get("resumable", False)
         run_created_at = run["created_at"].isoformat()
+        lg_logging.set_logging_context(
+            {
+                "run_id": str(run_id),
+                "run_attempt": attempt,
+                "thread_id": str(run.get("thread_id")),
+                "assistant_id": str(run.get("assistant_id")),
+                "graph_id": _get_graph_id(run),
+                "request_id": _get_request_id(run),
+            }
+        )
+
         await logger.ainfo(
             "Starting background run",
-            run_id=str(run_id),
-            run_attempt=attempt,
-            run_created_at=run_created_at,
             run_started_at=run_started_at.isoformat(),
             run_queue_ms=ms(run_started_at, run["created_at"]),
         )
@@ -141,7 +145,10 @@ async def worker(
                     on_checkpoint=on_checkpoint,
                     on_task_result=on_task_result,
                 )
-            await asyncio.wait_for(consume(stream, run_id), BG_JOB_TIMEOUT_SECS)
+            await asyncio.wait_for(
+                consume(stream, run_id, resumable),
+                BG_JOB_TIMEOUT_SECS,
+            )
             run_ended_at = datetime.now(UTC).isoformat()
             await logger.ainfo(
                 "Background run succeeded",
@@ -166,7 +173,6 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
-                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "timeout")
         except UserRollback as e:
@@ -183,7 +189,6 @@ async def worker(
                     run_started_at=run_started_at.isoformat(),
                     run_ended_at=run_ended_at,
                     run_exec_ms=ms(datetime.now(UTC), run_started_at),
-                    graph_id=_get_graph_id(run),
                 )
 
             except InFailedSqlTransaction as e:
@@ -221,7 +226,6 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
-                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "interrupted")
         except RETRIABLE_EXCEPTIONS as e:
@@ -237,7 +241,6 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
-                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "pending")
             raise
@@ -254,7 +257,6 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
-                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "error")
         set_auth_ctx(None, None)
@@ -275,21 +277,49 @@ async def worker(
         # let it bubble up and rollback db transaction, thus marking the run
         # as available to be picked up by another worker
 
-    return {
-        "checkpoint": checkpoint,
-        "status": status,
-        "run_started_at": run_started_at.isoformat(),
-        "run_ended_at": run_ended_at,
-        "run": run,
-        "exception": exception,
-        "webhook": webhook,
-    }
+    # If a stateful run succeeded but no checkoint was returned, it's likely because
+    # there was a retriable exception that resumed right at the end
+    if checkpoint is None and (not temporary) and webhook and status == "success":
+        await logger.ainfo(
+            "Fetching missing checkpoint for webhook",
+            run_id=str(run_id),
+            run_attempt=attempt,
+        )
+        try:
+            state_snapshot = await Threads.State.get(
+                conn, run["kwargs"]["config"], subgraphs=True
+            )
+            checkpoint = {"values": state_snapshot.values}
+        except Exception:
+            await logger.aerror(
+                "Failed to fetch missing checkpoint for webhook. Continuing...",
+                exc_info=True,
+                run_id=str(run_id),
+                run_attempt=attempt,
+            )
+
+    return WorkerResult(
+        checkpoint=checkpoint,
+        status=status,
+        exception=exception,
+        run=run,
+        webhook=webhook,
+        run_started_at=run_started_at.isoformat(),
+        run_ended_at=run_ended_at,
+    )
 
 
 def ms(after: datetime, before: datetime) -> int:
     return int((after - before).total_seconds() * 1000)
 
 
+def _get_request_id(run: Run) -> str | None:
+    try:
+        return run["kwargs"]["config"]["configurable"]["langgraph_request_id"]
+    except Exception:
+        return None
+
+
 def _get_graph_id(run: Run) -> str | None:
     try:
         return run["kwargs"]["config"]["configurable"]["graph_id"]
